<?php

namespace Drupal\templating;

/**
 * Created by PhpStorm.
 * User: USER
 * Date: 11/8/18
 * Time: 2:52 PM
 */
class Templating {

  public $logger;

  public function __construct() {
    $this->logger = \Drupal::logger('templating');
  }

  public function getTemplating($content) {
    $template_item = NULL;
    foreach ($content as $key => $item) {
      if ($content->{$key}->entity && $content->{$key}->entity instanceof \Drupal\node\Entity\Node) {
        $template_node = $content->{$key}->entity;
        if ($template_node->getType() == 'templating') {
          $template_item = $template_node;
        }
      }
    }
    return $template_item;
  }

  public function is_field_ready($entity, $field) {
    $bool = FALSE;
    if (is_object($entity) && $entity->hasField($field)) {
      $field_value = $entity->get($field)->getValue();
      if (!empty($field_value)) {
        $bool = TRUE;
      }
    }
    return $bool;
  }

  function get_field($entity, $field) {
    $bool = $this->is_field_ready($entity, $field);
    $result = [];
    if ($bool) {
      $items = $entity->get($field)->getValue();
      foreach ($items as $key => $item) {
        $result[] = $item['value'];
      }
    }
    return $result;

  }

  function create_file($template_name, $module_name, $type = 'block_content') {
    $module_handler = \Drupal::service('module_handler');
    $template_name = str_replace("_", "-", $template_name);
    $path_module = $module_handler->getModule($module_name)->getPath();
    $path_file =  $path_module . '/templates' . '/' . $template_name . '.html.twig';
    @chmod($path_file, 0777);
    if (file_exists($path_file)) {

    }
    else {
      $path_module = DRUPAL_ROOT . "/" . $path_module . "/templates";
      $content = $this->defaultContent($type);
      return $this->generateFile($path_module, $template_name, $content);
    }
  }

  function defaultContent($type) {
    $content = '{# add your content here #}';
    switch ($type) {
      case 'block_content':
        $content = "<div{{ attributes }}>" . "\n";
        $content = $content . "{{ title_prefix }}" . "\n";
        $content = $content . "{% if label %}";
        $content = $content . "<h2{{ title_attributes }}>{{ label }}</h2>" . "\n";
        $content = $content . "{% endif %}" . "\n";
        $content = $content . "  {{ title_suffix }}" . "\n";
        $content = $content . "  {% block content %}" . "\n";
        $content = $content . "    {{ content }}" . "\n";
        $content = $content . "  {% endblock %}" . "\n";
        $content = $content . "</div>";

        break;
      case 'node':
        $content = "<article{{ attributes }}>" . "\n";
        $content = $content . "{{ title_prefix }}" . "\n";
        $content = $content . "{% if not page %}" . "\n";
        $content = $content . "<h2{{ title_attributes }}>" . "\n";
        $content = $content . " <a href='{{url}}' rel='bookmark'>{{ label }}</a>" . "\n";
         $content = $content . " </h2>" . "\n";
      $content = $content . "{% endif %}" . "\n";
      $content = $content . "{{ title_suffix }}" . "\n";
      $content = $content . "{% if display_submitted %}" . "\n";
          $content = $content . "<footer>" . "\n";
            $content = $content . "  {{ author_picture }}" . "\n";
           $content = $content . "   <div{{ author_attributes }}>" . "\n";
              $content = $content . "    {% trans %}Submitted by {{ author_name }} on {{ date }}{% endtrans %}" . "\n";
                  $content = $content . "{{ metadata }}" . "\n";
             $content = $content . " </div>" . "\n";
          $content = $content . "</footer>" . "\n";
      $content = $content . " {% endif %}" . "\n";
      $content = $content . "<div{{ content_attributes }}>" . "\n";
        $content = $content . "  {{ content }}" . "\n";
     $content = $content . " </div>" . "\n";
      $content = $content . "</article>" . "\n";

    }
    return $content;
  }

  function generateFile($directory, $filename, $content) {
    $fileSystem = \Drupal::service('file_system');
    if (!is_dir($directory)) {
      if ($fileSystem->mkdir($directory, NULL, TRUE) === FALSE) {
        $this->logger->error('Failed to create directory ' . $directory);
        return FALSE;
      }
    }
    if (file_put_contents($directory . '/' . $filename . '.html.twig', $content) === FALSE) {
      $this->logger->error('Failed to write file ' . $filename);
      return FALSE;
    }
    if (@chmod($directory . '/' . $filename . '.html.twig', 0777)) {
      $this->logger->error('Failed to change permission file ' . $filename);
    }
    return TRUE;
  }

  function twigCompileInline($string) {
    $elements = [
      '#type' => 'inline_template',
      '#template' => $string,
    ];
    $renderer = \Drupal::service('renderer');

    try {
      return $renderer->renderPlain($elements);
    } catch (\Twig_Error_Syntax $e) {
      $this->logger->error('Failed to complied Templating content error: ' . $e->getMessage());
      return FALSE;
    } catch (\Exception $e) {
      $this->logger->error('Failed to complied Templating error: ' . $e->getMessage());
      return FALSE;
    }
  }

  function getFilePathTemplating($module_name, $template_name) {
    $module_handler = \Drupal::service('module_handler');
    $status = \Drupal::moduleHandler()->moduleExists($module_name);
    if ($status) {
      $path_module = $module_handler->getModule($module_name)->getPath();
      $template_name = str_replace("_", "-", $template_name);
      return DRUPAL_ROOT . "/" . $path_module . "/templates/" . $template_name . '.html.twig';

    }
    else {
      $this->logger->error('Module not find');
      return FALSE;
    }
  }

  function loadtemplateContent($module_name, $template_name) {
    $file = $this->getFilePathTemplating($module_name, $template_name);
    if (file_exists($file)) {
      return file_get_contents($file, FILE_USE_INCLUDE_PATH);
    }
    else {
      $this->logger->error('File  not find exist : ' . $file);
      return FALSE;
    }
  }

  function delete($node) {
    $template_name = $node->field_template_name->value;
    $module_name = $this->get_field($node, 'field_module_name');
    if (!empty($module_name)) {
      $file = $this->getFilePathTemplating($module_name[0], $template_name);
      if (is_writable($file) && @unlink($file)) {
      }
      else {
        $this->logger->error('File  not write : ' . $file);
      }
    }
  }


}